@@ -99,7 +99,7 @@ class MOC(object):
         >>> m = MOC(0, (1, 2))
         >>> m.add(1, (0,))
         >>> for (order, cells) in m:
-        ...     print(order, sorted(cells))
+        ...     print(str(order) + ' ' + str(sorted(cells)))
         0 [1, 2]
         1 [0]
         """
@@ -137,14 +137,116 @@ class MOC(object):
         [6, 7]
         """
 
-        if not isinstance(order, int):
-            raise TypeError('MOC order must be an integer')
-        elif not 0 <= order <= MAX_ORDER:
-            raise IndexError('MOC order must be in range 0-{0}'.format(
-                    MAX_ORDER))
+        order = self._validate_order(order)
 
         return frozenset(self._orders[order])
 
+    def __eq__(self, other):
+        """Equality test operator.
+
+        >>> MOC(1, (4, 5, 6, 7)) == MOC(0, (1,))
+        True
+        >>> MOC(2, (0, 1)) == MOC(0, (0,))
+        False
+        >>> MOC(1, (5, 6)) != MOC(1, (1, 2))
+        True
+        >>> MOC(2, (8, 9, 10, 11)) != MOC(1, (2,))
+        False
+        """
+
+        if not isinstance(other, MOC):
+            return NotImplemented
+
+        self.normalize()
+        other.normalize()
+
+        for order in range(0, MAX_ORDER + 1):
+            if self._orders[order] != other._orders[order]:
+                return False
+
+        return True
+
+    def __ne__(self, other):
+        """Inequality test operator.
+
+        Implemented via __eq__.
+        """
+
+        return not self.__eq__(other)
+
+    def __iadd__(self, other):
+        """In-place addition operator.
+
+        Updates the MOC to represent the union of itself and the other MOC.
+
+        >>> p = MOC(4, (5, 6))
+        >>> p += MOC(4, (7, 8))
+        >>> sorted(p[4])
+        [5, 6, 7, 8]
+        """
+
+        if not isinstance(other, MOC):
+            return NotImplemented
+
+        for (order, cells) in other:
+            self.add(order, cells)
+
+        return self
+
+    def __add__(self, other):
+        """Addition operator.
+
+        Returns a MOC which is the union of two MOCs.
+
+        >>> s = MOC(4, (5, 6)) + MOC(4, (7, 8))
+        >>> sorted(s[4])
+        [5, 6, 7, 8]
+        """
+
+        if not isinstance(other, MOC):
+            return NotImplemented
+
+        sum = self.copy()
+
+        sum += other
+
+        return sum
+
+    def __sub__(self, other):
+        """Subtraction operator.
+
+        Returns a MOC which is the copy of the first MOC with the
+        intersection with the second MOC removed.
+
+        >>> d = MOC(0, (0,)) - MOC(2, (15,))
+        >>> sorted(d[1])
+        [0, 1, 2]
+        >>> sorted(d[2])
+        [12, 13, 14]
+        """
+
+        if not isinstance(other, MOC):
+            return NotImplemented
+
+        diff = self.copy()
+        diff -= other
+
+        return diff
+
+    def __isub__(self, other):
+        """In-place subtraction operator.
+
+        Removes areas which overlap with the given MOC.
+        """
+
+        if not isinstance(other, MOC):
+            return NotImplemented
+
+        for (order, cells) in other:
+            self.remove(order, cells)
+
+        return self
+
     @property
     def order(self):
         """The highest order at which the MOC has cells.
@@ -277,31 +379,170 @@ class MOC(object):
 
         self._normalized = False
 
-        try:
-            order = int(order)
-        except ValueError as e:
-            raise TypeError('MOC order must be convertable to int')
+        order = self._validate_order(order)
 
-        if not 0 <= order <= MAX_ORDER:
-            raise ValueError('MOC order must be in range 0-{0}'.format(MAX_ORDER))
-
-        max_cells = self._order_num_cells(order)
         cell_set = set()
 
         for cell in cells:
-            try:
-                cell = int(cell)
-            except ValueError as e:
-                raise TypeError('MOC cell must be convertable to int')
-
-            if not 0 <= cell < max_cells:
-                raise ValueError('MOC cell order ' +
-                    '{0} must be in range 0-{1}'.format(order, max_cells - 1))
-
+            cell = self._validate_cell(order, cell)
             cell_set.add(cell)
 
         self._orders[order].update(cell_set)
 
+    def remove(self, order, cells):
+        """Remove cells at a given order from the MOC.
+        """
+
+        self._normalized = False
+
+        order = self._validate_order(order)
+
+        for cell in cells:
+            cell = self._validate_cell(order, cell)
+
+            self._compare_operation(order, cell, True, 'remove')
+
+    def clear(self):
+        """Clears all cells from a MOC.
+
+        >>> m = MOC(4, (5, 6))
+        >>> m.clear()
+        >>> m.cells
+        0
+        """
+
+        for order in range(0, MAX_ORDER + 1):
+            self._orders[order].clear()
+
+        self._normalized = True
+
+    def copy(self):
+        """Return a copy of a MOC.
+
+        >>> p = MOC(4, (5, 6))
+        >>> q = p.copy()
+        >>> sorted(q[4])
+        [5, 6]
+        """
+
+        copy = MOC(name=self.name, mocid=self.id,
+                origin=self.origin, moctype=self.type)
+
+        copy += self
+
+        return copy
+
+    def contains(self, order, cell, include_smaller=False):
+        """Test whether the MOC contains the given cell.
+
+        If the include_smaller argument is true then the MOC is considered
+        to include a cell if it includes part of that cell (at a higher
+        order).
+
+        >>> m = MOC(1, (5,))
+        >>> m.contains(0, 0)
+        False
+        >>> m.contains(0, 1, True)
+        True
+        >>> m.contains(0, 1, False)
+        False
+        >>> m.contains(1, 4)
+        False
+        >>> m.contains(1, 5)
+        True
+        >>> m.contains(2, 19)
+        False
+        >>> m.contains(2, 21)
+        True
+        """
+
+        order = self._validate_order(order)
+        cell = self._validate_cell(order, cell)
+
+        return self._compare_operation(order, cell, include_smaller, 'check')
+
+    def _compare_operation(self, order, cell, include_smaller, operation):
+        """General internal method for comparison-based operations.
+
+        This is a private method, and does not update the normalized
+        flag.
+        """
+
+        # Check for a larger cell (lower order) which contains the
+        # given cell.
+        for order_i in range(0, order):
+            shift = 2 * (order - order_i)
+            cell_i = cell >> shift
+
+            if cell_i in self._orders[order_i]:
+                if operation == 'check':
+                    return True
+                elif operation == 'remove':
+                    # Remove the cell and break it into its 4 constituent
+                    # cells.  Those which actually match the area we are
+                    # trying to remove will be removed at the next stage.
+                    self._orders[order_i].remove(cell_i)
+                    self.add(order_i + 1, range(cell_i << 2, (cell_i + 1) << 2))
+                elif operation == 'inter':
+                    return (order, (cell,))
+
+        # Check for the specific cell itself, but only after looking at larger
+        # cells because for the "remove" operation we may have broken up
+        # one of the large cells so that it subsequently matches.
+        if cell in self._orders[order]:
+            if operation == 'check':
+                return True
+            elif operation == 'remove':
+                self._orders[order].remove(cell)
+            elif operation == 'inter':
+                return (order, (cell,))
+
+        if include_smaller:
+            # Check for a smaller cell (higher order) which is part
+            # of the given cell.
+            for order_i in range(order + 1, MAX_ORDER + 1):
+                shift = 2 * (order_i - order)
+
+                cells = []
+
+                for cell_i in self._orders[order_i]:
+                    if (cell_i >> shift) == cell:
+                        if operation == 'check':
+                            return True
+                        elif operation == 'remove' or operation == 'inter':
+                            cells.append(cell_i)
+
+                if operation == 'remove':
+                    for cell_i in cells:
+                        self._orders[order_i].remove(cell_i)
+                elif operation == 'inter':
+                    return (order_i, cells)
+
+        if operation == 'check':
+            return False
+        else:
+            return None
+
+    def intersection(self, other):
+        """Returns a MOC representing the intersection with another MOC.
+
+        >>> p = MOC(2, (3, 4, 5))
+        >>> q = MOC(2, (4, 5, 6))
+        >>> i = p.intersection(q)
+        >>> sorted(i[2])
+        [4, 5]
+        """
+
+        inter = MOC()
+
+        for (order, cells) in other:
+            for cell in cells:
+                i = self._compare_operation(order, cell, True, 'inter')
+                if i is not None:
+                    inter.add(*i)
+
+        return inter
+
     def normalize(self, max_order=MAX_ORDER):
         """Ensure that the MOC is "well-formed".
 
@@ -324,8 +565,7 @@ class MOC(object):
         1
         """
 
-        if not 0 <= max_order <= MAX_ORDER:
-            raise ValueError('MOC order must be in range 0-{0}'.format(MAX_ORDER))
+        max_order = self._validate_order(max_order)
 
         # If the MOC is already normalized and we are not being asked
         # to reduce the order, then do nothing.
@@ -391,7 +631,38 @@ class MOC(object):
 
         self._normalized = True
 
-    def read(self, filename, filetype=None, include_meta=False):
+    def flattened(self, order=None, include_smaller=True):
+        """Return a flattened pixel collection at a single order."""
+
+        if order is None:
+            order = self.order
+        else:
+            order = self._validate_order(order)
+
+        # Start with the cells which are already at this order.
+        flat = set(self[order])
+
+        # Look at lower orders and expand them into this set.
+        # Based on the "map" algorithm from Appendix A of the
+        # MOC recommendation.
+        for order_i in range(0, order):
+            shift = 2 * (order - order_i)
+
+            for cell in self[order_i]:
+                flat.update(range(cell << shift, (cell + 1) << shift))
+
+        # Look at higher orders unless we have been told to exclude
+        # them.
+        if include_smaller:
+            for order_i in range(order + 1, MAX_ORDER + 1):
+                shift = 2 * (order_i - order)
+
+                for cell in self[order_i]:
+                    flat.add(cell >> shift)
+
+        return flat
+
+    def read(self, filename, filetype=None, include_meta=False, **kwargs):
         """Read data from the given file into the MOC object.
 
         The cell lists read from the file are added to the current
@@ -406,6 +677,10 @@ class MOC(object):
 
         Note that writing to FITS and JSON will cause the MOC
         to be normalized automatically.
+
+        Any additional keyword arguments (kwargs) are passed on to
+        the corresponding pymoc.io read functions (read_moc_fits,
+        read_moc_json or read_moc_ascii).
         """
 
         if filetype is not None:
@@ -415,24 +690,29 @@ class MOC(object):
 
         if filetype == 'fits':
             from .io.fits import read_moc_fits
-            read_moc_fits(self, filename, include_meta)
+            read_moc_fits(self, filename, include_meta, **kwargs)
 
         elif filetype == 'json':
             from .io.json import read_moc_json
-            read_moc_json(self, filename)
+            read_moc_json(self, filename, **kwargs)
 
         elif filetype == 'ascii' or filetype == 'text':
             from .io.ascii import read_moc_ascii
-            read_moc_ascii(self, filename)
+            read_moc_ascii(self, filename, **kwargs)
 
         else:
             raise ValueError('Unknown MOC file type {0}'.format(filetype))
 
-    def write(self, filename, filetype=None):
+    def write(self, filename, filetype=None, **kwargs):
         """Write the converage data in the MOC object to a file.
 
         The filetype can be given or left to be inferred as for the
         read method.
+
+        Any additional keyword arguments (kwargs) are passed on to
+        the corresponding pymoc.io write functions (write_moc_fits,
+        write_moc_json or write_moc_ascii).  This can be used, for
+        example, to set clobber=True when writing FITS files.
         """
 
         if filetype is not None:
@@ -442,15 +722,15 @@ class MOC(object):
 
         if filetype == 'fits':
             from .io.fits import write_moc_fits
-            write_moc_fits(self, filename)
+            write_moc_fits(self, filename, **kwargs)
 
         elif filetype == 'json':
             from .io.json import write_moc_json
-            write_moc_json(self, filename)
+            write_moc_json(self, filename, **kwargs)
 
         elif filetype == 'ascii' or filetype == 'text':
             from .io.ascii import write_moc_ascii
-            write_moc_ascii(self, filename)
+            write_moc_ascii(self, filename, **kwargs)
 
         else:
             raise ValueError('Unknown MOC file type {0}'.format(filetype))
@@ -490,3 +770,36 @@ class MOC(object):
         """Determine the number of possible cells for an order."""
 
         return 12 * 4 ** order
+
+    def _validate_order(self, order):
+        """Check that the given order is valid."""
+
+        try:
+            order = int(order)
+        except ValueError as e:
+            raise TypeError('MOC order must be convertable to int')
+
+        if not 0 <= order <= MAX_ORDER:
+            raise ValueError('MOC order must be in range 0-{0}'.format(
+                    MAX_ORDER))
+
+        return order
+
+    def _validate_cell(self, order, cell):
+        """Check that the given cell is valid.
+
+        The order is assumed already to have been validated.
+        """
+
+        max_cells = self._order_num_cells(order)
+
+        try:
+            cell = int(cell)
+        except ValueError as e:
+            raise TypeError('MOC cell must be convertable to int')
+
+        if not 0 <= cell < max_cells:
+            raise ValueError('MOC cell order ' +
+                '{0} must be in range 0-{1}'.format(order, max_cells - 1))
+
+        return cell
